function [u_g p_g]=diagnostic_core(m_dh,m_dhu,m_dv,m_ds,m_ss,m_bs,inputs);
%DIAGNOSTIC_CORE - compute the core velocity field 
%
%   Usage:
%      u_g=diagnostic_core(m_dh,m_dhu,m_dv,m_ds,m_ss,m_bs,inputs);
%

%recover parameters common to all solutions
debug=m_dhu.parameters.debug;
dim=m_dhu.parameters.dim;
ishutter=m_dhu.parameters.ishutter;
ismacayealpattyn=m_dh.parameters.ismacayealpattyn;
isstokes=m_ds.parameters.isstokes;

if ishutter,

	displaystring(debug,'\n%s',['computing surface slope (x and y derivatives)...']);
	slopex=diagnostic_core_linear(m_ss,'surface_slope_compute_x',inputs);
	slopey=diagnostic_core_linear(m_ss,'surface_slope_compute_y',inputs);

	if dim==3,
		displaystring(debug,'\n%s',['extruding slopes in 3d...']);
		slopex=SlopeExtrude(m_ss.elements,m_ss.nodes,m_ss.loads,m_ss.materials,slopex);
		slopey=SlopeExtrude(m_ss.elements,m_ss.nodes,m_ss.loads,m_ss.materials,slopey);
	end

	displaystring(debug,'\n%s',['computing slopes...']);
	inputs=add(inputs,'surfaceslopex',slopex,'doublevec',m_ss.parameters.numberofdofspernode,m_ss.parameters.numberofnodes);
	inputs=add(inputs,'surfaceslopey',slopey,'doublevec',m_ss.parameters.numberofdofspernode,m_ss.parameters.numberofnodes);

	displaystring(debug,'\n%s',['computing hutter velocities...']);
	u_g=diagnostic_core_linear(m_dhu,'diagnostic_hutter',inputs);

	displaystring(debug,'\n%s',['computing pressure according to MacAyeal...']);
	p_g=ComputePressure(m_dhu.elements,m_dhu.nodes,m_dhu.loads,m_dhu.materials,m_dhu.parameters);

	displaystring(debug,'\n%s',['update boundary conditions for macyeal pattyn using hutter results...']);
	if ismacayealpattyn,
		m_dh.y_g=u_g;
		[m_dh.ys m_dh.ys0]=Reducevectorgtos(m_dh.y_g,m_dh.nodesets);
	end

end

if ismacayealpattyn,

	displaystring(debug,'\n%s',['computing horizontal velocities...']);
	u_g=diagnostic_core_nonlinear(m_dh,inputs,'diagnostic_horiz');

	displaystring(debug,'\n%s',['computing pressure according to MacAyeal...']);
	p_g=ComputePressure(m_dh.elements,m_dh.nodes,m_dh.loads,m_dh.materials,m_dh.parameters);
end
	
if dim==3,

	displaystring(debug,'\n%s',['extruding horizontal velocities...']);
	u_g_horiz=VelocityExtrude(m_dh.elements,m_dh.nodes,m_dh.loads,m_dh.materials,u_g);

	displaystring(debug,'\n%s',['computing vertical velocities...']);
	inputs=add(inputs,'velocity',u_g,'doublevec',m_dh.parameters.numberofdofspernode,m_dh.parameters.numberofnodes);
	u_g_vert=diagnostic_core_linear(m_dv,inputs);

	displaystring(debug,'\n%s',['combining horizontal and vertical velocities...']);
	u_g=zeros(m_dh.parameters.numberofnodes*3,1);
	u_g(dofsetgen([1,2],3,m_dh.parameters.numberofnodes*3))=u_g_horiz;
	u_g(dofsetgen([3],3,m_dh.parameters.numberofnodes*3))=u_g_vert;

	displaystring(debug,'\n%s',['computing pressure according to Pattyn...']);
	p_g=ComputePressure(m_dh.elements,m_dh.nodes,m_dh.loads,m_dh.materials,m_dh.parameters);
	
	if isstokes,

		%"recondition" pressure 
		p_g=p_g/m_ds.parameters.stokesreconditioning;

		displaystring(debug,'\n%s',['computing bed slope (x and y derivatives)...']);
		slopex=diagnostic_core_linear(m_bs,'bed_slope_compute_x',inputs);
		slopey=diagnostic_core_linear(m_bs,'bed_slope_compute_y',inputs);
		slopex=SlopeExtrude(m_bs.elements,m_bs.nodes,m_bs.loads,m_bs.materials,slopex);
		slopey=SlopeExtrude(m_bs.elements,m_bs.nodes,m_bs.loads,m_bs.materials,slopey);

		inputs=add(inputs,'bedslopex',slopex,'doublevec',m_ss.parameters.numberofdofspernode,m_ss.parameters.numberofnodes);
		inputs=add(inputs,'bedslopey',slopey,'doublevec',m_ss.parameters.numberofdofspernode,m_ss.parameters.numberofnodes);
		
		inputs=add(inputs,'velocity',u_g,'doublevec',m_dh.parameters.numberofdofspernode,m_dh.parameters.numberofnodes);
		inputs=add(inputs,'pressure',p_g,'doublevec',1,m_dh.parameters.numberofnodes);

		displaystring(debug,'\n%s',['update boundary conditions for stokes using velocities previously computed...']);
		m_ds.y_g(dofsetgen([1,2],4,m_ds.parameters.gsize))=u_g;
		m_ds.y_g(dofsetgen([3],4,m_ds.parameters.gsize))=u_g_vert;
		[m_ds.ys m_ds.ys0]=Reducevectorgtos(m_ds.y_g,m_ds.nodesets);

		displaystring(debug,'\n%s',['computing stokes velocities and pressure ...']);
		u_g=diagnostic_core_nonlinear(m_ds,inputs,'diagnostic_stokes');
	
		%"decondition" pressure
		p_g=u_g(4:4:end)*m_dh.parameters.stokesreconditioning;
	end
end
